function [GAM,vars,xopt] = findbound(p)
% FINDBOUND --- Find a global lower bound (Shor) for a polynomial. 
%
% [GAM,VARS,XOPT] = findbound(P)
%
% P is a polynomial of even degree, whose global lower bound is to
% be computed.
%
% We compute the largest GAM such that
%
%        (P - GAM) is a sum of squares.
%
% In addition, a vector VARS containing all the variables in P, and
% the optimal argument XOPT (in the same ordering as VARS) will 
% be computed, such that by substituting VARS = XOPT into P 
% we obtain
%
%         P = GAM
%
% If no such optimal argument is found, the function will return 
% an empty XOPT.
%

% This file is part of SOSTOOLS - Sum of Squares Toolbox ver 1.00.
%
% Copyright (C) 2002  S. Prajna (1), A. Papachristodoulou (1), P. A. Parrilo (2)
% (1) Control and Dynamical Systems - California Institute of Technology,
%     Pasadena, CA 91125, USA.
% (2) Institut fur Automatik - ETH Zurich, CH-8092 Zurich, Switzerland.
%
% Send feedback to: sostools@cds.caltech.edu
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%

vars = findsym(p);
vars = sym(['[',vars,']']);

nvars = size(vars,2) ; 

% Can change, to call maple only once.
for var = 1:nvars;
    if rem(double(maple('degree',p,vars(var))),2) ;
    error(['Degree in ' char(vars(var)) ' should be even.']);
    end;
end;

syms gam;
prog = sosprogram(vars,gam);
prog = sosineq(prog,p-gam);
prog = sossetobj(prog,-gam);

[prog,info] = sossolve(prog);

xopt = [];

if (info.dinf==1)|(info.pinf==1)
    disp('No lower bound could be computed. Unbounded below?');
    GAM = '-inf';
else
    GAM = double(sosgetsol(prog,gam,16));
    % Returning the solution
    % The code here will only work in the rank one case.
    % Otherwise, we'll solve the f_i(x)=0 (efficiently)
    %
    % Find where the variables are (perhaps they're in the wrong order? Ask Stephen.
    ix = find(sum(prog.extravar.Z{1},2)==1) ;
    xopt = prog.solinfo.extravar.dual{1}(ix,1) ;
    % If the upper and lower bounds are close (absolute or relative), return them
    ach = subs(p,vars,xopt);
    if min(abs(ach/GAM-1),abs(ach-GAM)) > 1e-4 ; 
        xopt = [];
    end
end;

vars = vars.';